@@ -33,6 +33,11 @@
 #define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
   g_object_set_data (G_OBJECT (component), name, widget)
 
+void show_preferences(void);
+extern guint refresh_interval;
+extern guint rID;
+GtkWidget *refresh_spin;
+
 GtkWidget* create_main_window (void)
 {
     GtkWidget *window;
@@ -59,12 +64,12 @@ GtkWidget* create_main_window (void)
 
 	menubar = gtk_menu_bar_new();
 	gtk_widget_show( menubar );
-	gtk_box_pack_start(bbox1, menubar, FALSE, TRUE, 0 );
+	gtk_box_pack_start(GTK_BOX (bbox1), menubar, FALSE, TRUE, 0 );
 
 	/* build menu */
 	menu = gtk_menu_new();
 	item = gtk_menu_item_new_with_mnemonic( _("_File") );
-	gtk_menu_item_set_submenu( item, menu );
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), menu );
 	gtk_menu_shell_append( (GtkMenuShell*)menubar, item );
 
 	item = gtk_image_menu_item_new_from_stock( GTK_STOCK_QUIT, NULL );
@@ -75,12 +80,12 @@ GtkWidget* create_main_window (void)
 	gtk_menu_shell_append( (GtkMenuShell*)menubar, item );
 
 	menu = create_mainmenu();
-	gtk_menu_item_set_submenu( item, menu );
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), menu );
 
 	item = gtk_menu_item_new_with_mnemonic( _("_Help") );
 	gtk_menu_shell_append( (GtkMenuShell*)menubar, item );
 	menu = gtk_menu_new();
-	gtk_menu_item_set_submenu( item, menu );
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), menu );
 
     item= gtk_image_menu_item_new_from_stock ("gtk-about", NULL);
     gtk_widget_show (item);
@@ -92,10 +97,10 @@ GtkWidget* create_main_window (void)
 	/* window content */
     vbox1 = gtk_vbox_new (FALSE, 10);
     gtk_widget_show (vbox1);
-    gtk_box_pack_start( bbox1, vbox1, TRUE, TRUE, 0 );
+    gtk_box_pack_start( GTK_BOX(bbox1), vbox1, TRUE, TRUE, 0 );
     gtk_container_set_border_width (GTK_CONTAINER (vbox1), 6);
 
-    system_info_box = gtk_hbox_new (FALSE, 10);
+    system_info_box = gtk_hbox_new (TRUE, 10);
     gtk_widget_show (system_info_box);
     gtk_box_pack_start (GTK_BOX (vbox1), system_info_box, FALSE, TRUE, 0);
 
@@ -175,22 +180,25 @@ void create_list_store(void)
     gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_UNAME, compare_int_list_item, (void *)COLUMN_UNAME, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
+    cell_renderer = gtk_cell_renderer_text_new();
+    g_object_set(cell_renderer, "xalign", 1.0, NULL);
+
     column = gtk_tree_view_column_new_with_attributes(_("CPU%"), cell_renderer, "text", COLUMN_TIME, NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_TIME);
-    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_TIME, compare_string_list_item, (void *)COLUMN_TIME, NULL);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_TIME, compare_int_list_item, (void *)COLUMN_TIME, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
     column = gtk_tree_view_column_new_with_attributes(_("RSS"), cell_renderer, "text", COLUMN_RSS, NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_RSS);
-    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_RSS, compare_int_list_item, (void *)COLUMN_RSS, NULL);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_RSS, compare_size_list_item, (void *)COLUMN_RSS, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
     column = gtk_tree_view_column_new_with_attributes(_("VM-Size"), cell_renderer, "text", COLUMN_MEM, NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_MEM);
-    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_MEM, compare_int_list_item, (void *)COLUMN_MEM, NULL);
+    gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(list_store), COLUMN_MEM, compare_size_list_item, (void *)COLUMN_MEM, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
     column = gtk_tree_view_column_new_with_attributes(_("PID"), cell_renderer, "text", COLUMN_PID, NULL);
@@ -293,10 +301,12 @@ GtkWidget* create_mainmenu (void)
     GtkWidget *mainmenu;
     GtkWidget *info1;
     GtkWidget *trennlinie1;
+    GtkWidget *preferences1;
     GtkWidget *show_user_tasks1;
     GtkWidget *show_root_tasks1;
     GtkWidget *show_other_tasks1;
     GtkWidget *show_cached_as_free1;
+    GtkWidget *separator1;
     GtkAccelGroup *accel_group;
 
     accel_group = gtk_accel_group_new ();
@@ -323,6 +333,15 @@ GtkWidget* create_mainmenu (void)
     gtk_widget_show (show_cached_as_free1);
     gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), show_cached_as_free1);
 
+    separator1 = gtk_separator_menu_item_new();
+    gtk_widget_show(separator1);
+    gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), separator1);
+
+    preferences1 = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
+    g_signal_connect(G_OBJECT (preferences1), "activate", G_CALLBACK (show_preferences), NULL);
+    gtk_widget_show(preferences1);
+    gtk_menu_shell_append(GTK_MENU_SHELL(mainmenu), preferences1);
+
     g_signal_connect ((gpointer) show_user_tasks1, "toggled", G_CALLBACK (on_show_tasks_toggled), (void *)own_uid);
     g_signal_connect ((gpointer) show_root_tasks1, "toggled", G_CALLBACK (on_show_tasks_toggled), (void *)0);
     g_signal_connect ((gpointer) show_other_tasks1, "toggled", G_CALLBACK (on_show_tasks_toggled), (void *)-1);
@@ -339,6 +358,7 @@ void show_about_dialog(void)
     const gchar *authors[] =
     {
         "Hong Jen Yee <pcman.tw@gmail.com>",
+        "Jan Dlabal <dlabaljan@gmail.com>",
         _("LXTask is derived from Xfce4 Task Manager by:\n"
         "  * Johannes Zellner <webmaster@nebulon.de>"),
         NULL
@@ -357,7 +377,7 @@ void show_about_dialog(void)
     gtk_about_dialog_set_copyright ( (GtkAboutDialog*)about_dlg, _( "Copyright (C) 2008 LXDE team" ) );
     gtk_about_dialog_set_comments ( (GtkAboutDialog*)about_dlg, _( "Lightweight task manager for LXDE project" ) );
     gtk_about_dialog_set_license ( (GtkAboutDialog*)about_dlg, "LXTask\n\nCopyright (C) 2008 LXDE team\n\nmw program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nmw program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with mw program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA." );
-    gtk_about_dialog_set_website ( (GtkAboutDialog*)about_dlg, "http://lxde.sourceforge.net/" );
+    gtk_about_dialog_set_website ( (GtkAboutDialog*)about_dlg, "http://lxde.org/" );
     gtk_about_dialog_set_authors ( (GtkAboutDialog*)about_dlg, authors );
     gtk_about_dialog_set_translator_credits ( (GtkAboutDialog*)about_dlg, translators );
     /*gtk_window_set_transient_for( (GtkWindow*) about_dlg, GTK_WINDOW( mw ) );*/
@@ -370,51 +390,47 @@ void change_list_store_view(void)
 {
     gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_PPID), full_view);
     gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_MEM), full_view);
-    gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_RSS), full_view);
+    //gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_RSS), full_view);
+    gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_STATE), full_view);
+    gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_PRIO), full_view);
+
+    if(!show_root_tasks && !show_other_tasks)
+        gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_UNAME), full_view);
+    else
+        gtk_tree_view_column_set_visible (gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), COLUMN_UNAME), TRUE);
 }
 
 void fill_list_item(gint i, GtkTreeIter *iter)
 {
     if(iter != NULL)
     {
+        char buf[32];
         struct task *task = &g_array_index(task_array, struct task, i);
 
-        gchar *pid = g_strdup_printf("%i", task->pid);
-        gchar *ppid = g_strdup_printf("%i", task->ppid);
-        gchar *state = g_strdup_printf("%s", task->state);
-        gchar *size = g_strdup_printf("%i kB", task->size/*/1024*/);
-        gchar *rss = g_strdup_printf("%i kB", task->rss/*/1024*/);
-        gchar *name = g_strdup_printf("%s", task->name);
-        gchar *uname = g_strdup_printf("%s", task->uname);
-        gchar *time = g_strdup_printf("%0d%%", (guint)task->time_percentage);
-        gchar *prio = g_strdup_printf("%i", task->prio);    /* my change */
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_NAME, task->name, -1);
+        sprintf(buf,"%d",task->pid);
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PID, buf, -1);
+        sprintf(buf,"%d",task->ppid);
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PPID, buf, -1);
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_STATE, task->state, -1);
 
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_NAME, name, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PID, pid, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PPID, ppid, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_STATE, state, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_MEM, size, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_RSS, rss, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_UNAME, uname, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_TIME, time, -1);
-        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PRIO, prio, -1);    /* my change */
+        /* size */
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_MEM, size_to_string(buf, task->size * 1024), -1);
 
-        g_free(pid);
-        g_free(ppid);
-        g_free(state);
-        g_free(size);
-        g_free(rss);
-        g_free(name);
-        g_free(uname);
-        g_free(time);
-        g_free(prio);   /* my change */
+        /* rss */
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_RSS, size_to_string(buf, task->rss*1024), -1);
+
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_UNAME, task->uname, -1);
+        sprintf(buf,"%0d%%", (guint)task->time_percentage);
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_TIME, buf, -1);
+        sprintf(buf,"%d",task->prio);
+        gtk_tree_store_set(GTK_TREE_STORE(list_store), iter, COLUMN_PRIO, buf, -1);    /* my change */
     }
 }
 
 void add_new_list_item(gint i)
 {
         GtkTreeIter iter;
-
         gtk_tree_store_append(GTK_TREE_STORE(list_store), &iter, NULL);
 
         fill_list_item(i, &iter);
@@ -424,14 +440,13 @@ void refresh_list_item(gint i)
 {
     GtkTreeIter iter;
     gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list_store), &iter);
-    struct task task = g_array_index(task_array, struct task, i);
-
+    struct task *task = &g_array_index(task_array, struct task, i);
     while(valid)
     {
-        gchar *str_data = "";
+        gchar *str_data = NULL;
         gtk_tree_model_get(GTK_TREE_MODEL(list_store), &iter, COLUMN_PID, &str_data, -1);
 
-        if(task.pid == atoi(str_data))
+        if(str_data && task->pid == atoi(str_data))
         {
             g_free(str_data);
             fill_list_item(i, &iter);
@@ -450,10 +465,10 @@ void remove_list_item(gint pid)
 
     while(valid)
     {
-        gchar *str_data = "";
+        gchar *str_data = NULL;
         gtk_tree_model_get(GTK_TREE_MODEL(list_store), &iter, COLUMN_PID, &str_data, -1);
 
-        if(pid == atoi(str_data))
+        if(str_data && pid == atoi(str_data))
         {
             g_free(str_data);
             gtk_tree_store_remove(GTK_TREE_STORE(list_store), &iter);
@@ -470,7 +485,7 @@ gint compare_int_list_item(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIter
     gchar *s1 = "";
     gchar *s2 = "";
 
-    gint ret = 0;
+    gint ret;
 
     gtk_tree_model_get(model, iter1, column, &s1, -1);
     gtk_tree_model_get(model, iter2, column, &s2, -1);
@@ -480,16 +495,34 @@ gint compare_int_list_item(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIter
 
     if(s1 != NULL)
         i1 = atoi(s1);
-
     if(s2 != NULL)
         i2 = atoi(s2);
 
     ret = i2 - i1;
 
-    if(s1 != NULL)
-        g_free(s1);
-    if(s2 != NULL)
-        g_free(s2);
+    g_free(s1);
+    g_free(s2);
+
+    return ret;
+}
+
+gint compare_size_list_item(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIter *iter2, gpointer column)
+{
+    gchar *s1 = "";
+    gchar *s2 = "";
+
+    gint ret;
+
+    gtk_tree_model_get(model, iter1, column, &s1, -1);
+    gtk_tree_model_get(model, iter2, column, &s2, -1);
+
+    gint64 i1 = string_to_size(s1);
+    gint64 i2 = string_to_size(s2);
+
+    ret = i2 - i1;
+
+    g_free(s1);
+    g_free(s2);
 
     return ret;
 }
@@ -506,13 +539,9 @@ gint compare_string_list_item(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIt
 
     if(s1 != NULL && s2 != NULL)
         ret = strcmp(s2, s1);
-    else
-        ret = 0;
 
-    if(s1 != NULL)
-        g_free(s1);
-    if(s2 != NULL)
-        g_free(s2);
+    g_free(s1);
+    g_free(s2);
 
     return ret;
 }
@@ -520,14 +549,16 @@ gint compare_string_list_item(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIt
 /* change the task view (user, root, other) */
 void change_task_view(void)
 {
-    gtk_tree_store_clear(GTK_TREE_STORE(list_store));
     gint i = 0;
 
+    gtk_tree_store_clear(GTK_TREE_STORE(list_store));
+    change_list_store_view();
+
     for(i = 0; i < tasks; i++)
     {
-        struct task task = g_array_index(task_array, struct task, i);
+        struct task *task = &g_array_index(task_array, struct task, i);
 
-        if((task.uid == own_uid && show_user_tasks) || (task.uid == 0 && show_root_tasks) || (task.uid != own_uid && task.uid != 0 && show_other_tasks))
+        if((task->uid == own_uid && show_user_tasks) || (task->uid == 0 && show_root_tasks) || (task->uid != own_uid && task->uid != 0 && show_other_tasks))
             add_new_list_item(i);
     }
 
@@ -535,3 +566,36 @@ void change_task_view(void)
 }
 
 
+void apply_prefs()
+{
+    g_source_remove(rID);
+    refresh_interval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (refresh_spin));
+    rID = g_timeout_add(1000 * refresh_interval, (gpointer) refresh_task_list, NULL);
+}
+
+
+void show_preferences(void)
+{
+    GtkWidget *dlg;
+    GtkWidget *c_area;
+    GtkWidget *notebook = gtk_notebook_new();
+    GtkWidget *general_box = gtk_vbox_new(FALSE, 0);
+    GtkWidget *refresh_box = gtk_hbox_new(FALSE, 0);
+    refresh_spin = gtk_spin_button_new_with_range(1, 60, 1);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(refresh_spin), refresh_interval);
+
+    dlg = gtk_dialog_new_with_buttons(_("Preferences"), NULL, 0, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+    c_area = gtk_dialog_get_content_area(GTK_DIALOG (dlg));
+
+    gtk_box_pack_start_defaults(GTK_BOX (c_area), notebook);
+    gtk_notebook_append_page(GTK_NOTEBOOK (notebook), general_box, gtk_label_new(_("General")));
+    gtk_box_pack_start_defaults(GTK_BOX (refresh_box), gtk_label_new(_("Refresh rate (seconds):")));
+    gtk_box_pack_start_defaults(GTK_BOX (refresh_box), refresh_spin);
+    gtk_box_pack_start_defaults(GTK_BOX (general_box), refresh_box);
+
+    gtk_widget_show_all(notebook);
+
+    if (gtk_dialog_run(GTK_DIALOG (dlg)) == GTK_RESPONSE_ACCEPT)
+      apply_prefs();
+    gtk_widget_destroy(dlg);
+}
